#!/usr/bin/env python

import argparse
import csv


def main():
    parser = argparse.ArgumentParser(
        description="Analyze results of simulation run")
    parser.add_argument("infile",
                        type=argparse.FileType("rb"),
                        help="The result of simulation run (xls file)")
    args = parser.parse_args()

    result_reader = csv.DictReader(args.infile, delimiter="\t")
    current_state = dict()
    for i, row in enumerate(result_reader):
        if i == 0:
            current_state = row
            print("Initial state:" + str(current_state))
        else:
            changes = []
            for k, v in row.items():
                if k in ["Event", "Time", "Trajectory"]:
                    continue
                if v != current_state[k]:
                    if int(v):
                        changes.append("+" + k)
                    else:
                        changes.append("-" + k)
            print("Changes in step {} (trajectory {}): {}".format(i, row["Trajectory"], str(changes)))
            current_state = row


if __name__ == "__main__":
    main()
