#! /bin/bash

echo "Checking for consistency with stored results of bile acid simulation ..."


HOME_DIR=$(dirname $(readlink -f $0))
GOLDEN="$HOME_DIR/../golden_output/trimmed_BileAcid.xls"

if [ ! -f "$GOLDEN" ]; then
  echo "Couldn't find the golden solution"
  echo "(Should be in $GOLDEN)"
  exit 2
fi

WORKSPACE=validation_workspace
mkdir -p $WORKSPACE
if [ $# -eq 0 ]; then
  echo "running qsspn ..."
  $HOME_DIR/qsspn_runner.py --workspace $WORKSPACE
  cut -f1,4- $WORKSPACE/output.xls > $WORKSPACE/validation_trimmed_output.xls
  TRIMMED=$WORKSPACE/validation_trimmed_output.xls
else
  echo "checking $1 ..."
  cut -f1,4- $1 > $WORKSPACE/validation_trimmed_output.xls
fi
diff $WORKSPACE/validation_trimmed_output.xls $GOLDEN > /dev/null
if [ $? -eq 0 ]; then
  echo "Validation success"
else
  echo "Files differ! Inspect $WORKSPACE/validation_trimmed_output.xls for unexpected changes"
  exit 1
fi
