#include <map>
#include <string>
#include <stdlib.h>

#include "Cache.h"
#include <sstream>
#include <fstream>

// /////////////////////////////////////////////////////////////////////////////////////////////

Cache::Cache(std::size_t _max_size) : max_size(_max_size), current_size(0) {}

Cache::~Cache() {
  // delete map;
}

Cache::Cache(std::size_t _max_size, std::istream& in, std::string& outfilename)
    : max_size(_max_size), current_size(0), saveTo(outfilename) {
  std::string strBuf, dblBuf;
  std::cout << "Reading cache from file..." << std::endl;
  int numRead = 0;
  while(in.good()) {
    std::getline(in, strBuf, '\0');
    std::getline(in, dblBuf, '\0');
    std::istringstream dbStream(dblBuf);
    double value;
    dbStream >> value;
    setValue(strBuf, value);
    numRead ++;

  }
  std::cout << "Cache loaded " << numRead << " entries; size " << current_size << std::endl;
}

void Cache::serialize(std::ostream& out) {
  int numSaved =0;
  for(std::map<std::string, double>::iterator i = map.begin(); i != map.end(); i++) {
    out << i->first << '\0' << i->second << '\0';
    numSaved++;
  }
  std::cout << "Cache saved, " << numSaved << " entries, size " << (current_size / sizeof(char) / 1024) << "KB\n";
}

void Cache::clearIfTooBig() {
  if (current_size > max_size) {
    map.clear();
    current_size = 0;
    std::cout << "Cache cleared\n";
  }
}

bool Cache::isPresent(const std::string& s) {
  return map.count(s) == 1;
}

double Cache::getValue(const std::string& s) {
  return map[s];
}

void Cache::setValue(const std::string& s, double d) {
  clearIfTooBig();
  current_size += s.size();
  map[s] = d;
  if (last_save_size + 100* 1024 * sizeof(char) < current_size && saveTo.length() != 0) { // save whenever cache grows by 100 KB
    if (float(current_size) / max_size >= 0.9) { // serializing a full cache will cause it to be emptied just after loading
      saveTo = "";
    } else {
      std::ofstream out(saveTo.c_str());
      serialize(out);
      last_save_size = current_size;
    }
  }
}
