#include "MetabolismServer.h"

#include <iostream>
#include <functional>
#include <memory>

asio::ip::tcp::socket& MetabolismConnection::socket() {
  return socket_;
}

void MetabolismConnection::start() {
  socket_.async_receive(asio::buffer(buffer_, length),
                        std::bind(&MetabolismConnection::handle_read,
                                  shared_from_this(),
                                  std::placeholders::_1,
                                  std::placeholders::_2));
}

MetabolismConnection::MetabolismConnection(asio::io_service& io_service,
                                           PetriNet         *pn,
                                           Cache            *cache) :
  pn_(pn), cache_(cache), socket_(io_service), strand_(io_service) {}


void MetabolismConnection::handle_read(const asio::error_code& error,
                                       std::size_t             bytes_transfered)
{
  if (!error && (bytes_transfered > 0)) {
    std::string input_received(buffer_);


    // std::cout << "read: " << input_received;
    // *** Tutaj wstawic procedure liczaca ***

    double result;
    total_queries++;

    if (cache_->isPresent(input_received)) {
      cache_hits++;
      result = cache_->getValue(input_received);
    }
    else {
      result = pn_->runOptimization(input_received);
      cache_->setValue(input_received, result);
    }
    if (total_queries % 100 == 0) {
      cout << "cache hits: " << cache_hits << "/" << total_queries << endl;
    }

    asio::async_write(socket_,
                      asio::buffer(reinterpret_cast<const char *>(&result),
                                   sizeof(double)),
                      std::bind(&MetabolismConnection::handle_write,
                                shared_from_this(), std::placeholders::_1)
                      );
  } else {
    // metabolism_->dump();
  }
}

void MetabolismConnection::handle_write(const asio::error_code& error) {
  if (!error) {
    socket_.async_receive(asio::buffer(buffer_, length),
                          std::bind(&MetabolismConnection::handle_read,
                                    shared_from_this(),
                                    std::placeholders::_1,
                                    std::placeholders::_2));
  } else {
    // metabolism_->dump();
  }
}



typedef std::shared_ptr<MetabolismConnection>connection_ptr;

MetabolismServer::MetabolismServer(const std::string& address,
                                   const std::string& port,
                                   asio::io_service & io_service,
                                   std::size_t        thread_num,
                                   PetriNet          *pn,
                                   Cache             *cache) :
  pn_(pn),
  cache_(cache),
  thread_pool_size_(thread_num),
  io_service_(io_service),
  acceptor_(io_service_),
  new_connection_(MetabolismConnection::create(io_service_, pn_, cache_)) {
  asio::ip::tcp::resolver resolver(io_service_);
  asio::ip::tcp::resolver::query query(address, port);
  asio::ip::tcp::endpoint endpoint = *resolver.resolve(query);

  acceptor_.open(endpoint.protocol());
  acceptor_.set_option(asio::ip::tcp::acceptor::reuse_address(true));
  acceptor_.bind(endpoint);
  acceptor_.listen();
  acceptor_.async_accept(
    new_connection_->socket(), std::bind(
      &MetabolismServer::handleAccept, this,
      std::placeholders::_1));
}

void MetabolismServer::run() {
  std::vector<std::shared_ptr<asio::thread> >threads;

  for (std::size_t i = 0; i < thread_pool_size_; ++i) {
    std::shared_ptr<asio::thread>thread(new asio::thread(
                                          [ = ]() { io_service_.run(); }));
    threads.push_back(thread);
  }

  for (std::size_t i = 0; i < threads.size(); ++i) {
    threads[i]->join();
  }
}

void MetabolismServer::stop() {
  io_service_.stop();
}

void MetabolismServer::handleAccept(const asio::error_code& error) {
  if (!error) {
    new_connection_->start();
    new_connection_.reset(MetabolismConnection::create(io_service_, pn_, cache_));
    acceptor_.async_accept(
      new_connection_->socket(),
      std::bind(&MetabolismServer::handleAccept, this,
                std::placeholders::_1));
  }
}
