#ifndef __METABOLISM_SERVER_H__
#define __METABOLISM_SERVER_H__


#include "PetriNet.h"
#include "Cache.h"

#include <asio.hpp>
#include <functional>
#include <memory>


class MetabolismConnection: public std::enable_shared_from_this<MetabolismConnection> {
public:

  static MetabolismConnection* create(asio::io_service& io_service,
                                      PetriNet         *pn,
                                      Cache            *cache) {
    return new MetabolismConnection(io_service, pn, cache);
  }

  asio::ip::tcp::socket& socket();
  void                   start();

private:

  int cache_hits    = 0;
  int total_queries = 0;
  MetabolismConnection(asio::io_service& io_service,
                       PetriNet         *pn,
                       Cache            *cache);
  void handle_read(const asio::error_code& error,
                   std::size_t             bytes_transfered);
  void handle_write(const asio::error_code& error);
  asio::ip::tcp::socket socket_;
  static const std::size_t length = 10000;
  asio::io_service::strand strand_;
  char buffer_[length];
  PetriNet *pn_;
  Cache    *cache_;
};


typedef std::shared_ptr<MetabolismConnection>connection_ptr;

class MetabolismServer {
public:

  MetabolismServer(const std::string& address,
                   const std::string& port,
                   asio::io_service & io_service,
                   std::size_t        thread_num,
                   PetriNet          *pn,
                   Cache             *cache);
  void run();
  void stop();

private:

  void handleAccept(const asio::error_code& error);
  PetriNet   *pn_;
  Cache      *cache_;
  std::size_t thread_pool_size_;
  asio::io_service& io_service_;
  asio::ip::tcp::acceptor acceptor_;
  connection_ptr new_connection_;
};


#endif /* ifndef __METABOLISM_SERVER_H__ */
