#include "SingleStepHandler.h"


#include <iostream>


SingleStepHandler::SingleStepHandler(const string& qsspn_file,
                                     const string& ctrl_file,
                                     const string& srv_addr,
                                     const string& srv_port) :  config(srv_addr,
                                                                       srv_port)
{
  pn.readPetriNet(qsspn_file);
  pn.oneTimeSetup(ctrl_file, config);
  vector<double> doubleDefaultMarking = pn.getState();
  defaultMarking.resize(doubleDefaultMarking.size(), false);
  for(int i=0; i<doubleDefaultMarking.size(); i++) {
    defaultMarking[i] = doubleDefaultMarking[i];
  }
}

void SingleStepHandler::calculateStep(StepResult   & result,
                                      const Marking& input) {
  setMarking(input);
  runStep();
  extractResult(result);
}

void SingleStepHandler::setMarking(const Marking& input) {
  vector<double> pnState(input.marked.size());

  for (int i = 0; i < pnState.size(); i++) {
    pnState[i] = input.marked[i];
  }
  pn.setState(pnState);
}

void SingleStepHandler::runStep() {
  pn.preTrajectorySetup(); // This allocates a Trajectory object...
  pn.runSingleStep(config);
  pn.clearTrajectories();  // ... so this is required to avoid leaks
}

void SingleStepHandler::extractMarking(StepResult& result) {
  vector<double> pnState = pn.getState();
  vector<bool>   marking(pnState.size());

  for (int i = 0; i < pnState.size(); i++) {
    if (pnState[i] == 0) {
      marking[i] = false;
    } else if (pnState[i] == 1) {
      marking[i] = true;
    } else {
      std::cerr << "Non-binary state in Petri net: " << pnState[i] << std::endl;
      throw std::runtime_error("Non-binary state in Petri net");
    }
  }
  result.marked = marking;
}

void SingleStepHandler::extractResult(StepResult& result) {
  extractMarking(result);
  result.produced = pn.getTracker().getProductionSummary();
}

void SingleStepHandler::describeModel(ModelDescription& result) {
  result.defaultMarking = defaultMarking;
  result.placeNames = pn.getPlaceNames();
  result.trackedPlaceIndexes = pn.getTrackedIndexes();
}
