#include <asio.hpp>

#include "../PetriNet.h"
#include "../Cache.h"
#include "../MetabolismServer.h"
#include <memory>
#include <cstdlib>
#include <fstream>
#include <functional>
#include <iostream>
#include <vector>
using namespace std;

bool isJson(string path) {
  string json = ".json";
  if (path.length() < json.length()) return false;
  return (0 == path.compare(path.length() - json.length(), json.length(), json));
}

int main(int argc, char** argv) {
    if (argc != 3 && argc != 5)  {
        cout << VERSION << endl;
        cout << "USAGE FOR STANDALONE: qsspn qsspn_file ctrl_file" << endl;
        cout << "USAGE FOR CLIENT: qsspn qsspn_file ctrl_file address port" << endl;
        cout << "The qsspn_file file with extension .json is read as Json file" << endl;
        cout << "The qsspn file without .json extension is assumed to be in old qsspn format" << endl;
        cout << "Example cache size is 100000000" << endl;
        exit(1);
    }

    if (argc == 3) {//standalone
        PetriNet *pn = new PetriNet();
        string path(argv[1]);
        string path1(argv[2]);
        if (isJson(path)) {
            pn->readModel(path);
        } else {
            pn->readPetriNet(path);
        }

        ModeConfiguration::Config config;
        config.mode == ModeConfiguration::SERVER;
        pn->oneTimeSetup(path1, config);
        pn->hybridSimulation3(config);
        exit(0);
    } else {//client
        PetriNet *pn = new PetriNet();
        string path(argv[1]);
        string path1(argv[2]);
        string serverAddress(argv[3]);
        string serverPort(argv[4]);
        if (isJson(path)) {
            pn->readModel(path);
        } else {
            pn->readPetriNet(path);
        }
        ModeConfiguration::Config config(serverAddress, serverPort);
        pn->oneTimeSetup(path1, config);
        pn->hybridSimulation3(config);
        exit(0);
    }
    return 0;
}
