#include "../PetriNet.h"
#include "../SingleStepHandler.h"
#include <iostream>
#include "SingleStepProxy.h" // thrift generated


#include <thrift/concurrency/ThreadManager.h>
#include <thrift/concurrency/PosixThreadFactory.h>
#include <thrift/protocol/TBinaryProtocol.h>
#include <thrift/server/TNonblockingServer.h>
#include <thrift/transport/TServerSocket.h>
#include <thrift/transport/TBufferTransports.h>

using namespace::apache::thrift;
using namespace::apache::thrift::concurrency;
using namespace::apache::thrift::protocol;
using namespace::apache::thrift::server;
using namespace::apache::thrift::transport;

#define boost_ptr boost::shared_ptr


void print_usage(const string& self) {
  cout << "Usage: " << self;
  cout << " QSSPN_FILE CTRL_FILE SRV_ADDR SRV_PORT THRIFT_PORT\n"
          "Run QSSPN protobuf proxy.\n\n"
          "Listens for thrift single simulation step requests on port THRIFT_PORT. "
          "Forwards them to QSSPN server that is assumed to be "
          "listening at SRV_ADDR:SRV_PORT.\n";
}

int main(int argc, char **argv) {
  if (argc != 6) {
    cout << "Expected 6 parameters (got " << argc << ")\n";
    print_usage(string(argv[0]));
    return 1;
  }
  PetriNet pn;
  string   qsspn_file(argv[1]);
  string   ctrl_file(argv[2]);
  string   srv_addr(argv[3]);
  string   srv_port(argv[4]);
  int thrift_port = atoi(argv[5]);


  boost_ptr<SingleStepHandler> handler(new SingleStepHandler(qsspn_file,
                                                             ctrl_file,
                                                             srv_addr,
                                                             srv_port));
  boost_ptr<TProcessor> processor(new SingleStepProxyProcessor(handler));
  boost_ptr<TProtocolFactory> protocolFactory(new TBinaryProtocolFactory());

  /* Nonblocking server requires a thread pool, but we're using only one thread
     -- the bottleneck is the qsspn server anyway, plus the proxy is probably
     not thread-safe */
  boost_ptr<ThreadManager> tm = ThreadManager::newSimpleThreadManager(1);
  boost_ptr<PosixThreadFactory> threadFactory(new PosixThreadFactory());
  tm->threadFactory(threadFactory);
  tm->start();
  TNonblockingServer server(processor,
                            protocolFactory,
                            thrift_port,
                            tm);
  cout << "Starting server" << endl;
  server.serve();
  cout << "Server done" << endl;
  return 0;
}
