#include <iostream>

#include <thrift/protocol/TBinaryProtocol.h>
#include <thrift/transport/TSocket.h>
#include <thrift/transport/TTransportUtils.h>

#include "SingleStepProxy.h" // thrift generated

using namespace std;
using namespace apache::thrift;
using namespace apache::thrift::protocol;
using namespace apache::thrift::transport;


int main() {
  boost::shared_ptr<TTransport> socket(new TSocket("localhost", 9090));
  boost::shared_ptr<TTransport> transport(new TFramedTransport(socket));
  boost::shared_ptr<TProtocol>  protocol(new TBinaryProtocol(transport));
  SingleStepProxyClient client(protocol);
  transport->open();

  /* this is the initial marking from the BileAcidHomeostasis model */
  vector<bool> initialStateVec { 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,
                                 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0,
                                 0, 1, 1, 0, 1, 0, 0, 0, 1, 1,
                                 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0,
                                 0, 1, 1, 0, 0, 1, 0, 0, 1, 1,
                                 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
                                 0, 0, 1, 0, 1, 1, 0, 0, 0, 1,
                                 0, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 0,
                                 0, 1, 1, 1, 0, 0, 1, 0, 1, 0,
                                 1, 1, 0, 0, 1, 0, 1, 0, 0, 0 };
  int HA_idx = 87; /* index of BA_HOMEOSTASIS place */

  Marking marking;
  marking.marked = initialStateVec;
  StepResult result;


  int MAX_STEPS = 600;

  for (int i = 0; i < MAX_STEPS; i++) {
    client.calculateStep(result, marking);
    marking.marked = result.marked;

    if (result.marked[HA_idx]) {
      cout << "Homeostasis occured after " << (i + 1) << " steps.\n";
      return 0;
    }
  }
  cout << "Stopping simulation after " << MAX_STEPS <<
  " - no homeostasis occured\n";
  return 0;
}
