#include "EnzymesTokensEncoder.h"
#include "BinaryEnzymesTokensEncoder.h"
#include "SubstanceInterface.h"
#include "gtest/gtest.h"
#include "gmock/gmock.h"

#include <vector>


using namespace std;

using ::testing::Return;
using ::testing::_;

class MockSubstance : public SubstanceInterface {
public:
	  MOCK_METHOD0(getName, string());
	  MOCK_METHOD0(getTokens, int());
	  MOCK_METHOD1(setTokens,  void(int count));
	  MOCK_METHOD1(enzymeAction, bool(DriverInterface *gsmn));
	  MOCK_METHOD1(setEnzymeBounds, void(DriverInterface *gsmn));
	  MOCK_METHOD1(setEnzymeActivity, void(EnzymeActivity enz));
	  MOCK_METHOD1(allocateRWeights, void(int n));
	  MOCK_METHOD1(getRWeight, double(int n));
	  MOCK_METHOD2(setRWeight, void(int n, double w));
	  MOCK_METHOD0(getGsmnReaction, string());
	  MOCK_METHOD1(getGsmnReaction, string(int n));
	  MOCK_METHOD1(setGsmnReaction, void(string rn));
	  MOCK_METHOD0(getEnzymeActivity, EnzymeActivity());
	  MOCK_METHOD0(numberOfGsmnReactions, int());
};

MockSubstance* substanceReturningTokens(int count) {
	MockSubstance* substance = new MockSubstance;
	EXPECT_CALL(*substance, getTokens()).WillOnce(Return(count));
	return substance;
}

vector<SubstanceInterface*> substancesReturningTokens(vector<int> counts) {
	vector<SubstanceInterface*> substances;
	for (int i = 0; i < counts.size(); ++i) {
		substances.push_back(substanceReturningTokens(counts[i]));
	}
	return substances;
}

MockSubstance* substanceExpectingTokens(int count) {
	MockSubstance* substance = new MockSubstance;
	EXPECT_CALL(*substance, setTokens(count)).Times(1);
	return substance;
}

vector<SubstanceInterface*> substancesExpectingTokens(vector<int> counts) {
	vector<SubstanceInterface*> substances;
	for (int i = 0; i < counts.size(); ++i) {
		substances.push_back(substanceExpectingTokens(counts[i]));
	}
	return substances;
}

TEST(EnzymesTokensEncoder, Empty) {
	EnzymesTokensEncoder * encoder = new BinaryEnzymesTokensEncoder();
	int maximalEncoding = encoder->getMaximalEncoding();
    ASSERT_EQ(0, maximalEncoding);
}

TEST(EnzymesTokensEncoder, Single) {
	EnzymesTokensEncoder * encoder = new BinaryEnzymesTokensEncoder({NULL});
	int maximalEncoding = encoder->getMaximalEncoding();
    ASSERT_EQ(2, maximalEncoding);
}

TEST(EnzymesTokensEncoder, Many) {
	EnzymesTokensEncoder * encoder = new BinaryEnzymesTokensEncoder({NULL, NULL, NULL});
	int maximalEncoding = encoder->getMaximalEncoding();
    ASSERT_EQ(8, maximalEncoding);
}

TEST(EnzymesTokensEncoder, EncodeDecodeSingleSubstance) {
	vector<SubstanceInterface*> inSubstances = substancesReturningTokens({1});
	EnzymesTokensEncoder * encoder = new BinaryEnzymesTokensEncoder(inSubstances);

	vector<SubstanceInterface*> outSubstances = substancesExpectingTokens({1});
	EnzymesTokensEncoder * decoder = new BinaryEnzymesTokensEncoder(outSubstances);

	int encoding = encoder->encode();
	decoder->decodeAndSetSubstancesStates(encoding);
}

TEST(EnzymesTokensEncoder, EncodeDecodeManySubstances) {
	vector<SubstanceInterface*> inSubstances = substancesReturningTokens({0, 1});
	EnzymesTokensEncoder * encoder = new BinaryEnzymesTokensEncoder(inSubstances);

	vector<SubstanceInterface*> outSubstances = substancesExpectingTokens({0, 1});
	EnzymesTokensEncoder * decoder = new BinaryEnzymesTokensEncoder(outSubstances);

	int encoding = encoder->encode();
	decoder->decodeAndSetSubstancesStates(encoding);
}
