#!/bin/bash
usage_string="Usage: run.sh <cube_size> <steps_no> <comm_type> <neigh_type> where
              comm_type in {shuffle, forever, degrading, no_comm} and
              neighb_type in {immediate6, immediate26, none}
       example: run.sh 10 40 shuffle immediate6"

cube_size=${1}
steps_no=${2}
comm_type=${3}
neigh_type=${4}

if [ "$#" != 4 ]; then
  echo "Wrong number of arguments!"
  echo "$usage_string"
  exit -1;
fi

case $comm_type in
  shuffle|forever|degrading|no_comm)
    ;;
  *) 
    echo "Wrong comm_type specified!"
    echo "$usage_string"
    exit -1
  ;;
esac

case $neigh_type in
  immediate6|immediate26|none)
    ;;
  *) 
    echo "Wrong neigh_type specified!"
    echo "$usage_string"
    exit -1
  ;;
esac

if [[ "$comm_type" == "shuffle" && "$neigh_type" == "immediate6" ]]; then
  echo "shuffle immediate6"
  jar="/ab-qsspn/spark-precise_communication/target/scala-2.11/spark-precise_communication-assembly-0.1-SNAPSHOT.jar"
  params=""
elif [[ "$comm_type" == "shuffle" && "$neigh_type" == "immediate26" ]]; then
  echo "shuffle immediate26"
  jar="/ab-qsspn/spark-precise_communication/target/scala-2.11/spark-precise_communication-assembly-0.1-SNAPSHOT.jar"
  params="false false false true"
elif [[ "$comm_type" == "forever" && "$neigh_type" == "immediate6" ]]; then
  echo "forever immediate6"
  jar="/ab-qsspn/spark/target/scala-2.11/spark-assembly-0.1-SNAPSHOT.jar"
  params=""
elif [[ "$comm_type" == "forever" && "$neigh_type" == "immediate26" ]]; then
  echo "forever immediate26"
  jar="/ab-qsspn/spark/target/scala-2.11/spark-assembly-0.1-SNAPSHOT.jar"
  params="false false false true"
elif [[ "$comm_type" == "degrading" && "$neigh_type" == "immediate6" ]]; then
  echo "degrading immediate6"
  jar="/ab-qsspn/spark/target/scala-2.11/spark-assembly-0.1-SNAPSHOT.jar"
  params="true false false"
elif [[ "$comm_type" == "degrading" && "$neigh_type" == "immediate26" ]]; then
  echo "degrading immediate26"
  jar="/ab-qsspn/spark/target/scala-2.11/spark-assembly-0.1-SNAPSHOT.jar"
  params="true false false true"
elif [[ "$comm_type" == "no_comm" && "$neigh_type" == "none" ]]; then
  echo "no_comm none"
  jar="/ab-qsspn/spark/target/scala-2.11/spark-assembly-0.1-SNAPSHOT.jar"
  params="false false false false true"
else
  echo "we should no be here"
  exit -1;
fi

prefix="unbuffer spark-submit --master local[4]"
checkpoints_loc="/tmp/checkpoints"
spark_events_loc="/tmp/spark-events"
suffix=" 2>&1 > abqsspn.log"

rm -r -f $spark_events_loc
mkdir $spark_events_loc
rm -r -f $checkpoints_loc/*
#echo $prefix $jar $cube_size $steps_no $params $checkpoints_loc $suffix
$prefix $jar $cube_size $steps_no $params $checkpoints_loc $suffix
