package biosim.checkpoints

import biosim.PCellSt
import biosim.Simulation.{PrecWorld}
import org.apache.hadoop.fs.{FileSystem, LocatedFileStatus, Path}
import org.apache.spark.SparkContext
import org.apache.spark.graphx.{VertexId, VertexRDD}
import org.apache.spark.rdd.RDD
import org.log4s._

class CheckpointSaver(val checkpointDir: String)(implicit sc: SparkContext) {
  val log = getLogger

  def fileFor(stepNumber: Int): String = s"$checkpointDir/step-$stepNumber.obj"

  def save(world: PrecWorld, stepNumber: Int): Unit = {
    val plainRdd: RDD[(VertexId, PCellSt)] = world.vertices
    plainRdd.saveAsObjectFile(fileFor(stepNumber))
  }

  def listCheckpoints(): List[String] = {
    val fs = FileSystem.get(sc.hadoopConfiguration)
    val path = new Path(checkpointDir)
    if (!fs.isDirectory(path)) {
      return List.empty
    }
    val iter = fs.listFiles(new Path(checkpointDir), true)
    var files: List[LocatedFileStatus] = List.empty
    while(iter.hasNext) {
      files = iter.next() :: files
    }
    val names = files map (_.getPath.getParent.getName) filter (_.contains("step-"))
    names.distinct
  }

  def restoreVertices(): Option[(VertexRDD[PCellSt], Int)] = {
    val checkpoints = listCheckpoints()
    log info s"Found the following checkpoints: ${checkpoints.mkString(",")}"
    val numbers = checkpoints map (name => Integer.parseInt(name.split('-')(1).split('.')(0)))
    log info s"With numbers ${numbers.mkString(",")}"
    if (numbers.isEmpty) None else {
      val maxNum = numbers.max
      val plainRdd = sc.objectFile[(VertexId, PCellSt)](fileFor(maxNum))
      val vertRDD = VertexRDD(plainRdd)
      log info s"Created VertexRDD with ${vertRDD.count()} vertices"
      Some(vertRDD, maxNum)
    }
    
  }




}
