package biosim

import biosim.qsspn.ThriftConnection
import org.scalatest.FunSuite
import org.scalamock.scalatest.MockFactory
import thrift.ModelDescription

class FakeConnection(val desc : ModelDescription) extends ThriftConnection {
  override val modelDescription: ModelDescription = desc

  override def requestStep(cellState: Seq[Boolean]): StepResult = ???
}

class SimulationImplTest extends FunSuite with MockFactory {

  val fakeCell = PCellSt (
    id = -1,
    curState = Array(true),
    nextState =  Array(true),
    fstStep = false,
    canGive = true,
    offers = List(),
    requests = List(),
    request = None,
    offer = None
  )

  def getSimulation = {
    val fakeModelDescription = stub[ModelDescription]
    val substance: String = "substance"
    (fakeModelDescription.placeNames _).when().returns(Seq(substance))

    val fakeConnection = new FakeConnection(fakeModelDescription)

    new SimulationImpl(fakeConnection, substance, null)
  }

  test("testStepSystem") {}

  test("Cell should fulfill an incoming request directed to it") {
    val simulation = getSimulation
    val cellWithSingleIncomingRequest = fakeCell.copy(id = 2, requests = List((1, 2)))
    val newCell = simulation.pRunSingleCell(cellWithSingleIncomingRequest, null)
    assert(newCell.offer.getOrElse(-1) == 1)
  }

  test("Cell should not fulfill an incoming request not directed to it") {
    val simulation = getSimulation
    val cellWithSingleIncomingRequest = fakeCell.copy(requests = List((1, 2)))
    val newCell = simulation.pRunNextAttempt(cellWithSingleIncomingRequest, null)
    assert(newCell.offer.isEmpty)
  }

  test("Cell should not fulfill an incoming request if canGive is false") {
    val simulation = getSimulation
    val cellWithSingleIncomingRequest = fakeCell.copy(id = 2, requests = List((1, 2)), canGive = false)
    val newCell = simulation.pRunNextAttempt(cellWithSingleIncomingRequest, null)
    assert(newCell.offer.isEmpty)
  }

  test("Cell should set canGive to false after fulfilling a request") {
    val simulation = getSimulation
    val cellWithSingleIncomingRequest = fakeCell.copy(id = 2, requests = List((1, 2)))
    val newCell = simulation.pRunNextAttempt(cellWithSingleIncomingRequest, null)
    assert(!newCell.canGive)
  }

  test("Cell should mark substance as used after fulfilling request") {
    val simulation = getSimulation
    val cellWithSingleIncomingRequest = fakeCell.copy(id = 2, requests = List((1, 2)))
    val newCell = simulation.pRunNextAttempt(cellWithSingleIncomingRequest, null)
    assert(!newCell.nextState(simulation.bcastFieldIdx))
  }

  test("Cell that did not get a donation should not have substance marked") {
    val simulation = getSimulation
    val cellWithSingleIncomingRequest = fakeCell.copy(id = 2, requests = List((1, 2)))
    val newCell = simulation.pRunNextAttempt(cellWithSingleIncomingRequest, null)
    assert(!newCell.nextState(simulation.bcastFieldIdx))
  }

  test("testPRunFstAttempt") {}

}
